@@ -1,20 +1,8 @@
 
 @BOTTOM@
 
-/* Define to 1 if NLS is requested.  */
-#undef ENABLE_NLS
-
-/* Define as 1 if you have catgets and don't want to use GNU gettext.  */
-#undef HAVE_CATGETS
-
-/* Define as 1 if you have gettext and don't want to use GNU gettext.  */
-#undef HAVE_GETTEXT
-
-/* Define if your locale.h file contains LC_MESSAGES.  */
-#undef HAVE_LC_MESSAGES
-
-/* Define to 1 if you have the stpcpy function.  */
-#undef HAVE_STPCPY
+/* */
+#undef VERSION
 
 /* */
 #undef HAVE_TYPEOF
@@ -22,53 +10,62 @@
 /* */
 #undef HAVE_LONG_LONG
 
+/* */
+#undef HAVE_POINTER_COMPARISON_BUG
+
+/* */
+#undef HAVE_MAXINT_CONVERSION_BUG
+
+/* */
+#undef HAVE_SOCKLEN_T
+
+/* */
+#undef C_BIG_ENDIAN
+
+/* */
+#undef C_LITTLE_ENDIAN
+
+/* */
+#undef RENAME_OVER_EXISTING_FILES
+
+/* */
+#undef HAVE_STRLEN
+
+/* */
+#undef HAVE_STRCPY
+
+/* */
+#undef HAVE_STRCHR
+
+/* */
+#undef HAVE_STRRCHR
+
+/* */
+#undef HAVE_STRCMP
+
+/* */
+#undef HAVE_STRNCMP
+
+/* */
+#undef HAVE_STRCSPN
+
+/* */
+#undef HAVE_STRSTR
+
+/* */
+#undef HAVE_MEMMOVE
+
+/* */
+#undef HAVE_STRERROR
+
 /* */
 #undef HAVE_SIGSETJMP
 
 /* */
-#undef HAVE_CHMOD
+#undef HAVE_GETHOSTBYNAME
 
 /* */
-#undef HAVE_ALARM
-
-/* */
-#undef HAVE_CFMAKERAW
-
-/* */
-#undef HAVE_GETHOSTBYADDR
-
-/* */
-#undef HAVE_DHCP_OPTION
-
-/* */
-#undef HAVE_HERROR
-
-/* */
-#undef HAVE_BEGINTHREAD
-
-/* */
-#undef HAVE_MOUOPEN
-
-/* */
-#undef HAVE_READ_KBD
-
-/* */
-#undef HAVE_CLONE
-
-/* */
-#undef HAVE_PTHREADS
-
-/* */
-#undef HAVE_SSL
-
-/* */
-#undef HAVE_ZLIB
-
-/* */
-#undef HAVE_BZIP2
-
-/* */
-#undef X2
+#undef HAVE_GETHOSTBYNAME_BUG
 
 /* */
 #undef DEBUGLEVEL
@@ -76,6 +73,9 @@
 /* */
 #undef JS
 
+/* */
+#undef CHCEME_FLEXI_LIBU
+
 /* */
 #undef HAVE_PCRE
 
@@ -83,10 +83,31 @@
 #undef HAVE_REGEX
 
 /* */
-#undef CHCEME_FLEXI_LIBU
+#undef ENABLE_UTF8
 
-/* By PerM's ideological comittee */
-#undef VERSION
+/* */
+#undef HAVE_BEGINTHREAD
+
+/* */
+#undef HAVE_PTHREADS
+
+/* */
+#undef X2
+
+/* */
+#undef HAVE_SSL
+
+/* */
+#undef HAVE_OPENSSL
+
+/* */
+#undef HAVE_NSS
+
+/* */
+#undef HAVE_ZLIB
+
+/* */
+#undef HAVE_BZIP2
 
 /* */
 #undef G
@@ -100,36 +121,23 @@
 /* */
 #undef GRDRV_DIRECTFB
 
-/* */
-#undef GRDRV_PMSHELL
-
 /* */
 #undef GRDRV_X
 
 /* */
 #undef GRDRV_SDL
 
+/* */
+#undef GRDRV_PMSHELL
+
 /* */
 #undef GRDRV_ATHEOS
 
 /* */
 #undef DONT_INCLUDE_SETJMP
 
-/* Tiff by Brain */
-#undef HAVE_TIFF
-
 /* Jpeg by Clock */
 #undef HAVE_JPEG
 
-/* */
-#undef C_BIG_ENDIAN
-
-/* */
-#undef C_LITTLE_ENDIAN
-
-/* */
-#undef RENAME_OVER_EXISTING_FILES
-
-/* */
-#undef HAVE_SOCKLEN_T
-
+/* Tiff by Brain */
+#undef HAVE_TIFF
