@@ -33,22 +33,6 @@ struct f_data *init_formatted(struct document_options *opt)
 	return scr;
 }
 
-/* prototypes */
-void clear_formatted(struct f_data *);
-void r_xpand_spaces(struct part *, int);
-int split_line(struct part *);
-int put_chars_conv(struct part *, unsigned char *, int);
-void html_form_control(struct part *, struct form_control *);
-void add_frameset_entry(struct frameset_desc *, struct frameset_desc *, unsigned char *, unsigned char *, int, int, unsigned char);
-void *html_special(struct part *, int, ...);
-void do_format(char *, char *, struct part *, unsigned char *);
-void release_part(struct part *);
-void push_base_format(unsigned char *, struct document_options *, int);
-void add_srch_chr(struct f_data *, unsigned char, int, int, int);
-void sort_srch(struct f_data *);
-int get_srch(struct f_data *);
-
-
 void destroy_fc(struct form_control *fc)
 {
 	int i;
@@ -114,7 +98,7 @@ void free_additional_files(struct additional_files **a)
 	*a = NULL;
 }
 
-void clear_formatted(struct f_data *scr)
+static void clear_formatted(struct f_data *scr)
 {
 	int n;
 	int y;
@@ -241,8 +225,8 @@ struct rgb palette[] = {
 	{-1, -1, -1, 0}
 };
 
-/*struct rgb rgbcache = {0, 0, 0};
-int rgbcache_c = 0;
+/*static struct rgb rgbcache = {0, 0, 0};
+static int rgbcache_c = 0;
 
 static inline int find_nearest_color(struct rgb *r, int l)
 {
@@ -304,32 +288,32 @@ static inline int fg_color(int fg, int bg)
 	return fg;
 }
 
-#define XALIGN(x) (((x)+0x7f)&~0x7f)
+#define XALIGN(x) (((x)+0xf)&~0xf)
+#define YALIGN(x) (((x)+0x3ff)&~0x3ff)
 
-int nowrap = 0;
+static int nowrap = 0;
 
-static inline void xpand_lines(struct part *p, int y)
+static void xpand_lines(struct part *p, int y)
 {
 	/*if (y >= p->y) p->y = y + 1;*/
 	if (!p->data) return;
-	if (XALIGN((unsigned)y + (unsigned)p->yp) > MAXINT) overalloc();
+	if (YALIGN((unsigned)y + (unsigned)p->yp) > MAXINT) overalloc();
 	y += p->yp;
 	if (y >= p->data->y) {
 		int i;
-		if (XALIGN(y + 1) > XALIGN(p->data->y)) {
-			if (XALIGN((unsigned)y + 1) > MAXINT / sizeof(struct line)) overalloc();
-			p->data->data = mem_realloc(p->data->data, XALIGN(y+1)*sizeof(struct line));
+		if (YALIGN(y + 1) > YALIGN(p->data->y)) {
+			if (YALIGN((unsigned)y + 1) > MAXINT / sizeof(struct line)) overalloc();
+			p->data->data = mem_realloc(p->data->data, YALIGN(y+1)*sizeof(struct line));
 		}
 		for (i = p->data->y; i <= y; i++) {
 			p->data->data[i].l = 0;
-			p->data->data[i].c = p->bgcolor;
 			p->data->data[i].d = DUMMY;
 		}
 		p->data->y = i;
 	}
 }
 
-static inline void xpand_line(struct part *p, int y, int x)
+static void xpand_line(struct part *p, int y, int x)
 {
 	if (!p->data) return;
 	if (XALIGN((unsigned)x + (unsigned)p->xp) > MAXINT) overalloc();
@@ -347,14 +331,15 @@ static inline void xpand_line(struct part *p, int y, int x)
 			if (XALIGN((unsigned)x + 1) > MAXINT / sizeof(chr)) overalloc();
 			p->data->data[y].d = mem_realloc(p->data->data[y].d, XALIGN(x+1)*sizeof(chr));
 		}
-		for (i = p->data->data[y].l; i <= x; i++)
-			p->data->data[y].d[i] = (p->data->data[y].c << 11) | ' ';
-		p->data->data[y].c = p->bgcolor;
+		for (i = p->data->data[y].l; i <= x; i++) {
+			p->data->data[y].d[i].at = p->bgcolor << 3;
+			p->data->data[y].d[i].ch = ' ';
+		}
 		p->data->data[y].l = i;
 	}
 }
 
-void r_xpand_spaces(struct part *p, int l)
+static void r_xpand_spaces(struct part *p, int l)
 {
 	unsigned char *c;
 	if ((unsigned)l >= MAXINT) overalloc();
@@ -375,28 +360,37 @@ static inline void xpand_spaces(struct part *p, int l)
 #define X(x) (p->xp + (x))
 #define Y(y) (p->yp + (y))
 
-static inline void set_hchar(struct part *p, int x, int y, unsigned c)
+static inline void set_hchar(struct part *p, int x, int y, unsigned ch, unsigned at)
 {
+	chr *cc;
 	xpand_lines(p, y);
 	xpand_line(p, y, x);
-	POS(x, y) = c;
+	cc = &POS(x, y);
+	cc->ch = ch;
+	cc->at = at;
 }
 
-static inline void set_hchars(struct part *p, int x, int y, int xl, unsigned c)
+static inline void set_hchars(struct part *p, int x, int y, int xl, unsigned ch, unsigned at)
 {
+	chr *cc;
 	xpand_lines(p, y);
 	xpand_line(p, y, x+xl-1);
-	for (; xl; xl--, x++) POS(x, y) = c;
+	cc = &POS(x, y);
+	for (; xl; xl--) {
+		cc->ch = ch;
+		cc->at = at;
+		cc++;
+	}
 }
 
-void xset_hchar(struct part *p, int x, int y, unsigned c)
+void xset_hchar(struct part *p, int x, int y, unsigned ch, unsigned at)
 {
-	set_hchar(p, x, y, c);
+	set_hchar(p, x, y, ch, at);
 }
 
-void xset_hchars(struct part *p, int x, int y, int xl, unsigned c)
+void xset_hchars(struct part *p, int x, int y, int xl, unsigned ch, unsigned at)
 {
-	set_hchars(p, x, y, xl, c);
+	set_hchars(p, x, y, xl, ch, at);
 }
 
 void xxpand_lines(struct part *p, int y)
@@ -409,20 +403,45 @@ void xxpand_line(struct part *p, int y, int x)
 	xpand_line(p, y, x);
 }
 
-static inline void set_hline(struct part *p, int x, int y, int xl, unsigned char *d, unsigned c, int spc)
+static inline void set_hline(struct part *p, int x, int y, int xl, unsigned char *d, unsigned at)
 {
+	chr *cc;
 	xpand_lines(p, y);
 	xpand_line(p, y, x+xl-1);
-	if (spc) xpand_spaces(p, x+xl-1);
+	xpand_spaces(p, x+xl-1);
+	cc = NULL;
+	if (p->data) cc = &POS(x, y);
 	for (; xl; xl--, x++, d++) {
-		if (spc) p->spaces[x] = *d == ' ';
-		if (p->data) POS(x, y) = *d | c;
+		p->spaces[x] = *d == ' ';
+		if (p->data) {
+			cc->ch = *d;
+			cc->at = at;
+			cc++;
+		}
 	}
 }
 
-int last_link_to_move;
-struct tag *last_tag_to_move;
-struct tag *last_tag_for_newline;
+static inline void set_hline_uni(struct part *p, int x, int y, int xl, char_t *d, unsigned at)
+{
+	chr *cc;
+	xpand_lines(p, y);
+	xpand_line(p, y, x+xl-1);
+	xpand_spaces(p, x+xl-1);
+	cc = NULL;
+	if (p->data) cc = &POS(x, y);
+	for (; xl; xl--, x++, d++) {
+		p->spaces[x] = *d == ' ';
+		if (p->data) {
+			cc->ch = *d;
+			cc->at = at;
+			cc++;
+		}
+	}
+}
+
+static int last_link_to_move;
+static struct tag *last_tag_to_move;
+static struct tag *last_tag_for_newline;
 
 static inline void move_links(struct part *p, int xf, int yf, int xt, int yt)
 {
@@ -490,7 +509,7 @@ static inline void shift_chars(struct part *p, int y, int s)
 	if ((unsigned)l > MAXINT / sizeof(chr)) overalloc();
 	a = mem_alloc(l * sizeof(chr));
 	memcpy(a, &POS(0, y), l * sizeof(chr));
-	set_hchars(p, 0, y, s, (p->data->data[y].c << 11) | ' ');
+	set_hchars(p, 0, y, s, ' ', p->bgcolor << 3);
 	copy_chars(p, s, y, l, a);
 	mem_free(a);
 	move_links(p, 0, y, s, y);
@@ -504,9 +523,9 @@ static inline void del_chars(struct part *p, int x, int y)
 
 #define rm(x) ((x).width - (x).rightmargin > 0 ? (x).width - (x).rightmargin : 0)
 
-void line_break(struct part *);
+static void line_break(void *);
 
-int split_line(struct part *p)
+static int split_line(struct part *p)
 {
 	int i;
 	/*if (!p->data) goto r;*/
@@ -531,7 +550,7 @@ int split_line(struct part *p)
 	if (i + par_format.rightmargin > p->x) p->x = i + par_format.rightmargin;
 	if (p->data) {
 #ifdef DEBUG
-		if ((POS(i, p->cy) & 0xff) != ' ') internal("bad split: %c", (char)POS(i, p->cy));
+		if (POS(i, p->cy).ch != ' ') internal("bad split: %c", (char)POS(i, p->cy).ch);
 #endif
 		move_chars(p, i+1, p->cy, par_format.leftmargin, p->cy+1);
 		del_chars(p, i, p->cy);
@@ -546,7 +565,7 @@ int split_line(struct part *p)
 	return 1 + (p->cx == -1);
 }
 
-void align_line(struct part *p, int y)
+static void align_line(struct part *p, int y)
 {
 	int na;
 	if (!p->data) return;
@@ -595,95 +614,85 @@ unsigned char *last_image = NULL;
 struct form_control *last_form = NULL;
 struct js_event_spec *last_js_event = NULL;
 
-int nobreak;
+static int nobreak;
 
 struct conv_table *convert_table;
 
-void put_chars(struct part *, unsigned char *, int);
-
-#define CH_BUF	256
-
-int put_chars_conv(struct part *p, unsigned char *c, int l)
+static void put_chars(void *p_, unsigned char *c, int l)
 {
-	static char buffer[CH_BUF];
-	int bp = 0;
-	int pp = 0;
-	int total = 0;
-	if (format.attr & AT_GRAPHICS) {
-		put_chars(p, c, l);
-		return l;
-	}
-	if (!l) put_chars(p, NULL, 0);
-	while (pp < l) {
-		unsigned char *e;
-		if (c[pp] < 128 && c[pp] != '&') {
-			putc:
-			buffer[bp++] = c[pp++];
-			if (bp < CH_BUF) continue;
-			goto flush;
-		}
-		if (c[pp] != '&') {
-			struct conv_table *t;
-			int i;
-			if (!convert_table) goto putc;
-			t = convert_table;
-			i = pp;
-			decode:
-			if (!t[c[i]].t) {
-				e = t[c[i]].u.str;
-			} else {
-				t = t[c[i++]].u.tbl;
-				if (i >= l) goto putc;
-				goto decode;
-			}
-			pp = i + 1;
-		} else {
-			int i = pp + 1;
-			if (d_opt->plain & 1) goto putc;
-			while (i < l && c[i] != ';' && c[i] != '&' && c[i] > ' ') i++;
-			if (!(e = get_entity_string(&c[pp + 1], i - pp - 1, d_opt->cp))) goto putc;
-			pp = i + (i < l && c[i] == ';');
-		}
-		if (!e[0]) continue;
-		if (!e[1]) {
-			buffer[bp++] = e[0];
-			if (bp < CH_BUF) continue;
-			flush:
-			e = "";
-			goto flush1;
-		}
-		while (*e) {
-			buffer[bp++] = *(e++);
-			if (bp < CH_BUF) continue;
-			flush1:
-			put_chars(p, buffer, bp);
-			total += bp;
-			bp = 0;
-		}
-	}
-	if (bp) put_chars(p, buffer, bp);
-	total += bp;
-	return total;
-}
+	struct part *p = p_;
 
-void put_chars(struct part *p, unsigned char *c, int l)
-{
 	static struct text_attrib_beginning ta_cache = { -1, { 0, 0, 0, 0 }, { 0, 0, 0, 0 }, 0, 0 };
 	static int bg_cache;
 	static int fg_cache;
+	char_t *uni_c = DUMMY;
 
 	int bg, fg;
 	int i;
 	struct link *link;
 	struct point *pt;
+	int ll;
+
 	if (l < 0) overalloc();
 	/*printf("%d-", p->cx);for (i=0; i<l; i++) printf("%c", c[i]); printf("-\n");sleep(1);*/
-	while (par_format.align != AL_NO && p->cx == -1 && l && *c == ' ') c++, l--;
+	while (p->cx <= par_format.leftmargin && l && *c == ' ' && par_format.align != AL_NO) c++, l--;
 	if (!l) return;
+	if (p->cx < par_format.leftmargin) p->cx = par_format.leftmargin;
 	if (c[0] != ' ' || (c[1] && c[1] != ' ')) {
 		last_tag_for_newline = (void *)&p->data->tags;
 	}
-	if (p->cx < par_format.leftmargin) p->cx = par_format.leftmargin;
+#ifdef ENABLE_UTF8
+	if (is_cp_special(d_opt->cp)) {
+		int pl;
+		unsigned char *cc;
+		if (p->utf8_part_len) {
+			unsigned char *q;
+			next_utf_byte:
+			if ((*c & 0xc0) != 0x80)
+				goto bad_utf;
+			p->utf8_part[p->utf8_part_len++] = *c;
+			p->utf8_part[p->utf8_part_len] = 0;
+			c++;
+			l--;
+			q = p->utf8_part;
+			if (!get_utf_8(&q)) {
+				if (p->utf8_part_len == sizeof(p->utf8_part) - 1)
+					goto bad_utf;
+				if (l) goto next_utf_byte;
+				return;
+			}
+			pl = p->utf8_part_len;
+			p->utf8_part_len = 0;
+			put_chars(p, p->utf8_part, pl);
+		}
+		bad_utf:
+		p->utf8_part_len = 0;
+		uni_c = mem_alloc(l * sizeof(char_t));
+		ll = 0;
+		cc = c;
+		next_utf_char:
+		pl = utf8chrlen(*cc);
+		if (cc - c + pl > l) {
+			memcpy(p->utf8_part, cc, p->utf8_part_len = l - (cc - c));
+			goto utf_done;
+		} else {
+			if (!pl) cc++;
+			else {
+				GET_UTF_8(cc, uni_c[ll]);
+				ll++;
+			}
+			if (cc < c + l) goto next_utf_char;
+		}
+		utf_done:
+		if (!ll) {
+			mem_free(uni_c);
+			return;
+		}
+	} else
+#endif
+	{
+		ll = l;
+	}
 	if (last_link || last_image || last_form || format.link || format.image 
 	|| format.form || format.js_event || last_js_event
 	) goto process_link;
@@ -692,11 +701,20 @@ void put_chars(struct part *p, unsigned char *c, int l)
 	if (memcmp(&ta_cache, &format, sizeof(struct text_attrib_beginning))) goto format_change;
 	bg = bg_cache, fg = fg_cache;
 	end_format_change:
-	if (p->cx == par_format.leftmargin && *c == ' ' && par_format.align != AL_NO) c++, l--;
 	if (p->y < p->cy + 1) p->y = p->cy + 1;
-	if (nowrap && p->cx + l > rm(par_format)) return;
-	set_hline(p, p->cx, p->cy, l, c, (((fg&0x08)<<3)|(bg<<3)|(fg&0x07))<<8, 1);
-	p->cx += l;
+	if (nowrap && p->cx + ll > rm(par_format)) {
+		mem_free(uni_c);
+		return;
+	}
+#ifdef ENABLE_UTF8
+	if (is_cp_special(d_opt->cp)) {
+		set_hline_uni(p, p->cx, p->cy, ll, uni_c, ((fg&0x08)<<3)|(bg<<3)|(fg&0x07));
+	} else
+#endif
+	{
+		set_hline(p, p->cx, p->cy, l, c, ((fg&0x08)<<3)|(bg<<3)|(fg&0x07));
+	}
+	p->cx += ll;
 	nobreak = 0;
 	if (par_format.align != AL_NO)
 		while (p->cx > rm(par_format) && p->cx > par_format.leftmargin) {
@@ -710,7 +728,8 @@ void put_chars(struct part *p, unsigned char *c, int l)
 			align_line(p, p->cy - 1);
 			nobreak = x - 1;
 		}
-	if ((p->xa += l) - (c[l-1] == ' ' && par_format.align != AL_NO) + par_format.leftmargin + par_format.rightmargin > p->xmax) p->xmax = p->xa - (c[l-1] == ' ' && par_format.align != AL_NO) + par_format.leftmargin + par_format.rightmargin;
+	if ((p->xa += ll) - (c[l-1] == ' ' && par_format.align != AL_NO) + par_format.leftmargin + par_format.rightmargin > p->xmax) p->xmax = p->xa - (c[l-1] == ' ' && par_format.align != AL_NO) + par_format.leftmargin + par_format.rightmargin;
+	mem_free(uni_c);
 	return;
 
 	/* !!! WARNING: THE FOLLOWING CODE IS SHADOWED IN HTML_GR.C */
@@ -799,12 +818,12 @@ void put_chars(struct part *p, unsigned char *c, int l)
 		link->sel_color = ((fg & 8) << 3) | (fg & 7) | (bg << 3);
 		link->n = 0;
 		set_link:
-		if ((unsigned)link->n + (unsigned)l > MAXINT / sizeof(struct point)) overalloc();
-		pt = mem_realloc(link->pos, (link->n + l) * sizeof(struct point));
+		if ((unsigned)link->n + (unsigned)ll > MAXINT / sizeof(struct point)) overalloc();
+		pt = mem_realloc(link->pos, (link->n + ll) * sizeof(struct point));
 		link->pos = pt;
-		for (i = 0; i < l; i++) pt[link->n + i].x = X(p->cx) + i,
-					pt[link->n + i].y = Y(p->cy);
-		link->n += l;
+		for (i = 0; i < ll; i++) pt[link->n + i].x = X(p->cx) + i,
+					 pt[link->n + i].y = Y(p->cy);
+		link->n += ll;
 	}
 	goto no_l;
 
@@ -822,8 +841,9 @@ void put_chars(struct part *p, unsigned char *c, int l)
 		goto end_format_change;
 }
 
-void line_break(struct part *p)
+static void line_break(void *p_)
 {
+	struct part *p = p_;
 	struct tag *t;
 	/*printf("-break-\n");*/
 	if (p->cx + par_format.rightmargin > p->x) p->x = p->cx + par_format.rightmargin;
@@ -837,7 +857,7 @@ void line_break(struct part *p)
 	if (!p->data) goto e;
 	/*move_links(p, p->cx, p->cy, 0, p->cy + 1);*/
 	xpand_lines(p, p->cy + 1);
-	if (p->cx > par_format.leftmargin && LEN(p->cy) > p->cx - 1 && (POS(p->cx-1, p->cy) & 0xff) == ' ') del_chars(p, p->cx-1, p->cy), p->cx--;
+	if (p->cx > par_format.leftmargin && LEN(p->cy) > p->cx - 1 && POS(p->cx-1, p->cy).ch == ' ') del_chars(p, p->cx-1, p->cy), p->cx--;
 	/*if (LEN(p->cy) > p->x) p->x = LEN(p->cy);*/
 	if (p->cx > 0) align_line(p, p->cy);
 	if (p->data) for (t = last_tag_for_newline; t && (void *)t != &p->data->tags; t = t->prev) {
@@ -852,7 +872,7 @@ void line_break(struct part *p)
 int g_ctrl_num;
 
 /* SHADOWED IN g_html_form_control */
-void html_form_control(struct part *p, struct form_control *fc)
+static void html_form_control(struct part *p, struct form_control *fc)
 {
 	if (!p->data) {
 		/*destroy_fc(fc);
@@ -884,7 +904,7 @@ void html_form_control(struct part *p, struct form_control *fc)
 	add_to_list(p->data->forms, fc);
 }
 
-void add_frameset_entry(struct frameset_desc *fsd, struct frameset_desc *subframe, unsigned char *name, unsigned char *url, int marginwidth, int marginheight, unsigned char scrolling)
+static void add_frameset_entry(struct frameset_desc *fsd, struct frameset_desc *subframe, unsigned char *name, unsigned char *url, int marginwidth, int marginheight, unsigned char scrolling)
 {
 	if (fsd->yp >= fsd->y) return;
 	fsd->f[fsd->xp + fsd->yp * fsd->x].subframe = subframe;
@@ -954,8 +974,9 @@ void html_process_refresh(struct f_data *f, unsigned char *url, int time)
 	f->refresh_seconds = time;
 }
 
-void *html_special(struct part *p, int c, ...)
+static void *html_special(void *p_, int c, ...)
 {
+	struct part *p = p_;
 	va_list l;
 	unsigned char *t;
 	struct form_control *fc;
@@ -1013,10 +1034,10 @@ void *html_special(struct part *p, int c, ...)
 	return NULL;
 }
 
-void do_format(char *start, char *end, struct part *part, unsigned char *head)
+static void do_format(char *start, char *end, struct part *part, unsigned char *head)
 {
 	pr(
-	parse_html(start, end, (int (*)(void *, unsigned char *, int)) put_chars_conv, (void (*)(void *)) line_break, (void *(*)(void *, int, ...)) html_special, part, head);
+	parse_html(start, end, put_chars, line_break, html_special, part, head);
 	/*if ((part->y -= line_breax) < 0) part->y = 0;*/
 	) {};
 }
@@ -1037,13 +1058,13 @@ struct table_cache_entry {
 	struct part p;
 };
 
-struct list_head table_cache = { &table_cache, &table_cache };
+static struct list_head table_cache = { &table_cache, &table_cache };
 
 #define TC_HASH_SIZE	4096
 
-struct table_cache_entry *table_cache_hash[TC_HASH_SIZE];
+static struct table_cache_entry *table_cache_hash[TC_HASH_SIZE];
 
-void free_table_cache()
+void free_table_cache(void)
 {
 	struct table_cache_entry *tce;
 	foreach(tce, table_cache) {
@@ -1101,14 +1122,14 @@ struct part *format_html_part(unsigned char *start, unsigned char *end, int alig
 	last_form = NULL;
 	last_js_event = NULL;
 	nobreak = 1;
-	p = mem_alloc(sizeof(struct part));
-	p->x = p->y = 0;
+	p = mem_calloc(sizeof(struct part));
+	/*p->x = p->y = 0;*/
 	p->data = data;
 	p->xp = xs; p->yp = ys;
-	p->xmax = p->xa = 0;
+	/*p->xmax = p->xa = 0;*/
 	p->bgcolor = find_nearest_color(&par_format.bgcolor, 8);
 	p->spaces = DUMMY;
-	p->spl = 0;
+	/*p->spl = 0;*/
 	p->link_num = link_num;
 	init_list(p->uf);
 	html_stack_dup();
@@ -1176,12 +1197,12 @@ struct part *format_html_part(unsigned char *start, unsigned char *end, int alig
 	return p;
 }
 
-void release_part(struct part *p)
+static void release_part(struct part *p)
 {
 	mem_free(p);
 }
 
-void push_base_format(unsigned char *url, struct document_options *opt, int frame)
+static void push_base_format(unsigned char *url, struct document_options *opt, int frame)
 {
 	struct html_element *e;
 	if (html_stack.next != &html_stack) {
@@ -1222,7 +1243,7 @@ struct conv_table *get_convert_table(unsigned char *head, int to, int def, int *
 	unsigned char *a, *b;
 	unsigned char *p = head;
 	while (from == -1 && p && (a = parse_http_header(p, "Content-Type", &p))) {
-		if ((b = parse_header_param(a, "charset"))) {
+		if ((b = parse_header_param(a, "charset", 0))) {
 			from = get_cp_index(b);
 			mem_free(b);
 		}
@@ -1267,6 +1288,7 @@ void really_format_html(struct cache_entry *ce, unsigned char *start, unsigned c
 	head = init_str(), hdl = 0;
 	if (ce->head) add_to_str(&head, &hdl, ce->head);
 	scan_http_equiv(start, end, &head, &hdl, &t, d_opt->plain ? NULL : &bg, d_opt->plain ? NULL : &bgcolor, &screen->js_event);
+	if (d_opt->plain) *t = 0;
 	convert_table = get_convert_table(head, screen->opt.cp, screen->opt.assume_cp, &screen->cp, &screen->ass, screen->opt.hard_assume);
 	screen->opt.real_cp = screen->cp;
 	i = d_opt->plain; d_opt->plain = 0;
@@ -1294,25 +1316,6 @@ void really_format_html(struct cache_entry *ce, unsigned char *start, unsigned c
 			if (screen->y > h) h = screen->y;
 			g_x_extend_area(rp->root, w, h);
 			screen->root = (struct g_object *)rp->root, rp->root = NULL;
-			screen->hsb = 0, screen->vsb = 0;
-			screen->hsbsize = 0, screen->vsbsize = 0;
-			if (screen->opt.scrolling == SCROLLING_YES) {
-				screen->hsb = 1, screen->vsb = 1;
-			} else if (screen->opt.scrolling == SCROLLING_AUTO) {
-				x:
-				if (!screen->hsb && screen->x > screen->opt.xw - screen->vsb * G_SCROLL_BAR_WIDTH) {
-					screen->hsb = 1;
-					goto x;
-				}
-				if (!screen->vsb && screen->y > screen->opt.yw - screen->hsb * G_SCROLL_BAR_WIDTH) {
-					screen->vsb = 1;
-					goto x;
-				}
-			}
-			if (screen->hsb) screen->hsbsize = screen->opt.xw - screen->vsb * G_SCROLL_BAR_WIDTH;
-			if (screen->vsb) screen->vsbsize = screen->opt.yw - screen->hsb * G_SCROLL_BAR_WIDTH;
-			if (screen->hsb < 0) screen->hsb = 0;
-			if (screen->vsb < 0) screen->vsb = 0;
 			g_release_part(rp);
 			mem_free(rp);
 			get_parents(screen, screen->root);
@@ -1339,7 +1342,7 @@ void really_format_html(struct cache_entry *ce, unsigned char *start, unsigned c
 		internal("html stack not empty after operation");
 		init_list(html_stack);
 	}
-	screen->bg = 007 << 8; /* !!! FIXME */
+	screen->bg = 007; /* !!! FIXME */
 	sort_links(screen);
 	current_f_data = NULL;
 	d_opt = &dd_opt;
@@ -1390,7 +1393,7 @@ void copy_opt(struct document_options *o1, struct document_options *o2)
 	o1->framename = stracpy(o2->framename);
 }
 
-void add_srch_chr(struct f_data *f, unsigned char c, int x, int y, int nn)
+static void add_srch_chr(struct f_data *f, unsigned c, int x, int y, int nn)
 {
 	int n = f->nsearch;
 	if (c == ' ' && (!n || f->search[n - 1].c == ' ')) return;
@@ -1412,7 +1415,7 @@ void add_srch_chr(struct f_data *f, unsigned char c, int x, int y, int nn)
 	debug("!");
 }*/
 
-void sort_srch(struct f_data *f)
+static void sort_srch(struct f_data *f)
 {
 	int i;
 	int *min, *max;
@@ -1432,12 +1435,12 @@ void sort_srch(struct f_data *f)
 	mem_free(max);
 }
 
-static inline int is_spc(chr c)
+static inline int is_spc(chr *cc)
 {
-	return (unsigned char)c <= ' ' || c & ATTR_FRAME;
+	return cc->ch <= ' ' || cc->at & ATTR_FRAME;
 }
 
-int get_srch(struct f_data *f)
+static int get_srch(struct f_data *f)
 {
 	struct node *n;
 	int cnt = 0;
@@ -1450,8 +1453,8 @@ int get_srch(struct f_data *f)
 		for (y = n->y; y < ym && y < f->y; y++) {
 			int ns = 1;
 			for (x = n->x; x < xm && x < f->data[y].l; x++) {
-				unsigned char c = f->data[y].d[x];
-				if (is_spc(f->data[y].d[x])) c = ' ';
+				unsigned c = f->data[y].d[x].ch;
+				if (is_spc(&f->data[y].d[x])) c = ' ';
 				if (c == ' ' && ns) continue;
 				c = charset_upcase(c, f->opt.cp);
 				if (ns) {
@@ -1464,7 +1467,7 @@ int get_srch(struct f_data *f)
 					       else cnt++;
 				else {
 					int xx;
-					for (xx = x + 1; xx < xm && xx < f->data[y].l; xx++) if (!is_spc(f->data[y].d[xx])) goto ja_uz_z_toho_programovani_asi_zcvoknu;
+					for (xx = x + 1; xx < xm && xx < f->data[y].l; xx++) if (!is_spc(&f->data[y].d[xx])) goto ja_uz_z_toho_programovani_asi_zcvoknu;
 					xx = x;
 					ja_uz_z_toho_programovani_asi_zcvoknu:
 					if (!cc) add_srch_chr(f, ' ', x, y, xx - x);
